 /*-------------------------------------------------------------------*
  * BOXANNO SAS    Annotate a scatter plot with univariate boxplots   *
  *-------------------------------------------------------------------*
  * Author:   Michael Friendly            <FRIENDLY@YORKVM1>          *
  * Created:  20 Apr 1988 11:32:44                                    *
  * Revised:  17 May 1990 09:51:13                                    *
  * Version:  1.5                                                     *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *-------------------------------------------------------------------*/
 
 /*------------------------------------------------------------------*
  | BOXAXIS macro - create an annotate dataset to draw a boxplot for |
  |         ONE axis in a scatterplot. Can be used with Proc GPLOT   |
  |         or Proc G3D scatterplots.                                |
  |         This macro just creates the annotate dataset. It is up to|
  |         the user to call the appropriate plot procedure.         |
  |   e.g., Proc GPLOT data= ;                                       |
  |             Plot Y * X / annotate= ... ;                         |
  *------------------------------------------------------------------*/
%macro BOXAXIS(
       data=_LAST_,          /* Input dataset                     */
       out=_DATA_,           /* Output ANNOTATE dataset           */
       var=,                 /* Variable to be plotted            */
       baxis=x,              /* Axis on which it goes- X, Y, or Z */
       oaxis=y,              /* The other axis in the plot        */
       paxis=z,              /* The 3rd axis (ignored in GPLOT)   */
       boxwidth=4,           /* width of box in data percent      */
       pos=98);              /* position of box on OAXIS 0<POS<100*/
 
%local bx by bz;             /* macro symbols for annotate x,y,z  */
%if ( &baxis = %str() or &oaxis = %str() ) %then %do;
    %put ERROR: Box (BAXIS) and other (OAXIS) axes must be specified;
%end;
%else %do;
   %if (&baxis=&oaxis) %then %do;
       %put ERROR: BAXIS (&BAXIS) cannot be the same as OAXIS (&OAXIS);
   %end;
%end;
 /*----------------------------------*
  | Find median & quartiles          |
  *----------------------------------*/
proc univariate data=&data noprint;
    var &var;
    output out=quartile
           n=n q1=q1 q3=q3 median=median qrange=iqr mean=mean;
run;
 /*-----------------------------------------------*
  | Find outside & farout points                  |
  *-----------------------------------------------*/
data plotdat;
    set &data;
    if _n_=1 then set quartile;
    retain q1 q3 iqr;
    keep &var outside;
    outside=1;
    if &var < (q1-1.5*iqr) or &var > (q3+1.5*iqr)
       then outside=2;
    if &var < (q1-3.0*iqr) or &var > (q3+3.0*iqr)
       then outside=3;
run;
 /*----------------------------------------------------*
  |  Whiskers go from quartiles to most extreme values |
  |  which are *NOT* outside.                          |
  *----------------------------------------------------*/
data whis;
    set plotdat;
    if outside = 1;
proc univariate data=whis noprint;
    var &var;
    output out=whisk min=lo_whisk max=hi_whisk;
run;
data boxfile;
    merge quartile whisk;
proc print data=boxfile;
 /*-----------------------------------------------*
  | Annotate data set to draw boxes & whiskers    |
  *-----------------------------------------------*/
%let bx = &oaxis;
%let by = &baxis;
%let bz = &paxis;
data &out;
    set boxfile;
    drop n lo_whisk hi_whisk q1 q3 iqr median mean
         center halfwid;
    length function $8 text $8;
    halfwid= &boxwidth / 2;
%if ( &pos > 50 ) %then %do;
    center= &pos - halfwid; %end;
%else %do;
    center= &pos + halfwid; %end;
    &bx.sys = '1';         /* data percentage coordinates for 'other' */
    &by.sys = '2';         /* data value coordinates for box axis     */
%if ( &paxis ^= %str() ) %then %do;
    &bz.sys = '1';         /* data percentage coordinates for 3rd axis*/
    &bz     = 1  ;
%end;
&bx =center-halfwid       ; &by = q1;       dot=1 ; link out; * box   ;
&bx =center+halfwid       ; &by = q1;       dot=21; link out;
&bx =center+halfwid       ; &by = q3;       dot=22; link out;
&bx =center-halfwid       ; &by = q3;       dot=23; link out;
&bx =center-halfwid       ; &by = q1;       dot=24; link out; * box   ;
 
&bx =center-halfwid       ; &by = median  ; dot=3 ; link out; * median;
&bx =center+halfwid       ; &by = median  ; dot=4 ; link out;
 
&bx =center               ; &by = q1      ; dot=5 ; link out; * lo     ;
&bx =center               ; &by = lo_whisk; dot=6 ; link out; * whisker;
&bx =center               ; &by = q3      ; dot=7 ; link out; * hi     ;
&bx =center               ; &by = hi_whisk; dot=8 ; link out; * whisker;
&bx =center-halfwid/2     ; &by = lo_whisk; dot=9 ; link out;
&bx =center+halfwid/2     ; &by = lo_whisk; dot=10; link out;
&bx =center-halfwid/2     ; &by = hi_whisk; dot=11; link out;
&bx =center+halfwid/2     ; &by = hi_whisk; dot=12; link out;
&bx =center               ; &by = mean    ; dot=13; link out;
    return;
 
out:
   select;
      when (dot=1 | dot=3 | dot=5 | dot=7 | dot=9 | dot=11) do;
         line = .;
         function = 'MOVE';          output;
      end;
      when (dot=4 | dot=6 | dot=8 | dot=10 | dot=12
          | dot=21| dot=22| dot=23| dot=24 ) do;
         if dot=6 | dot=8
            then line = 3;
            else line = 1;
         function = 'DRAW';          output;
      end;
      when (dot = 13) do;
         text = 'STAR';
         function = 'SYMBOL';        output;
      end;
      otherwise;
   end;
   return;
run;
%mend boxaxis;
 
 /*---------------------------------------------------------*
  | BOXANNO macro - creates annotate dataset for both X & Y |
  *---------------------------------------------------------*/
%macro boxanno(
     data=_last_,           /* Data set to be plotted  */
     xvar=,                 /* Horizontal variable     */
     yvar=,                 /* Vertical variable       */
     out=boxanno            /* Output annotate dataset */
     );
 
%boxaxis(
     data=&data, var=&xvar,
     baxis=x,    oaxis=y,    out=xanno);
 
%boxaxis(
     data=&data, var=&yvar,
     baxis=y,    oaxis=x,    out=yanno);
 /*----------------------------------------*
  |  Concatenate the two annotate datasets |
  *----------------------------------------*/
data &out;
     set xanno yanno;
%mend boxanno;
