 /*-------------------------------------------------------------------*
  * BOXPLOT SAS    SAS/Graph Box and Whisker plot                     *
  *                                                                   *
  * This SAS macro constructs and plots side-by-side Box and whisker  *
  * plots for ONE quantitative variable classified by ONE OR MORE     *
  * grouping (CLASS) variables. The CLASS variables may be character  *
  * or numeric.                                                       *
  *                                                                   *
  * The box for each group shows the location of the median, mean and *
  * quartiles. Whisker lines extend to the most extreme observations  *
  * which are no more than 1.5*IQR beyond the quartiles. Observations *
  * beyond the whiskers are plotted individually.                     *
  *                                                                   *
  * Optional NOTCHES are drawn to show approximate 95% confidence     *
  * intervals for each group median. Other options are provided to    *
  * connect group medians, draw box widths proportional to sample     *
  * size, and allow formatted labels for both variables.              *
  * References:                                                       *
  *   Olmstead, A. "Box Plots using SAS/Graph Software", SAS SUGI,    *
  *     1985, 888-894.                                                *
  *   McGill, R., Tukey, J.W., & Larsen, W. "Variations of Box Plots",*
  *     American Statistician, 1978, 32, 12-16.                       *
  *-------------------------------------------------------------------*
  * Author:   Michael Friendly            <FRIENDLY@YORKVM1>          *
  * Created:  12 Apr 1988 10:19:15                                    *
  * Revised:  3 Sep 1991 13:45:19                                     *
  * Version:  1.3                                                     *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *-------------------------------------------------------------------*/
                               /* Description of Parameters:         */
%macro BOXPLOT(                /* -------------------------          */
       data=_LAST_,            /* Input dataset                      */
       class=,                 /* Grouping variable(s)               */
       var=,                   /* Ordinate variable                  */
       id=,                    /* Observation ID variable            */
       width=.5,               /* Box width as proportion of maximum */
       notch=0,                /* =0|1, 1=draw notched boxes         */
       connect=0,              /* =0 or line style to connect medians*/
       f=0.5,                  /* Notch depth, fraction of halfwidth */
       fn=1,                   /* Box width proportional to &FN      */
       varfmt=,                /* Format for ordinate variable       */
       classfmt=,              /* Format for class variable(s)       */
       varlab=,                /* Label for ordinate variable        */
       classlab=,              /* Label for class variable(s)        */
       yorder=,                /* Tick marks, range for ordinate     */
       anno=,                  /* Addition to ANNOTATE set           */
       out=boxstat,            /* Output data set: quartiles, etc.   */
       name=BOXPLOT            /* Name for graphic catalog entry     */
       );
 
options nonotes;
%let _DSN_ = %upcase(&DATA);
%if &classlab = %str() %then %let classlab = &class;
%let CLASS = %upcase(&CLASS);
proc sort data=&DATA;
    by &CLASS;
run;
%let clvars = %nvar(&class);
 
 /*----------------------------------------*
  | Determine if &CLASS is char or numeric |
  *----------------------------------------*/
%let cltype=;
proc contents data=&DATA out=work noprint;
data _NULL_;
     length label2 $40;
     set work;
     if name="&CLASS"
        then if type=1 then call symput('CLTYPE', 'NUM');
                       else call symput('CLTYPE', 'CHAR');
 
     *-- find length of variable label and set y label angle --;
     %if &varlab ^= %str() %then
        %str( label2 = "&varlab"; );
     %else
        %str( if name="&VAR" then label2=label; );
     if length(label2) <=8
        then call symput('YANGLE','');
        else call symput('YANGLE','a=90 r=0');
run;              /* Run required here */
 
 /*----------------------------------------------------------------*
  |  If there are more than one class variables or class variable  |
  |  is CHAR, create a numeric class variable, XCLASS. XCLASS      |
  |  numbers the groups from 1,...number-of-groups. It is up to    |
  |  the user to supply a format to associate proper group labels  |
  |  with the XCLASS value.                                        |
  *----------------------------------------------------------------*/
%if ( &cltype=CHAR or &clvars > 1 ) %then %do;
   %let lclass = %scan( &CLASS, &clvars );
   data work;
      set &DATA;
      by &CLASS;
      if (first.&LCLASS) then xclass + 1;
      %if &cltype=CHAR and &clvars=1 and &classfmt=%str() %then
         %do;
         call symput('val'||left(put( xclass, 2. )), trim(&class) );
         %end;
 
   run;
   %let KLASS = xclass;
   %let data  = work;
   run;
%end;
%else %let KLASS = &CLASS;
 
 /*------------------------------------------------*
  | Determine number of groups & quartiles of each |
  *------------------------------------------------*/
proc means noprint data=&data;
    var &KLASS;
    output out=_grsum_ min=grmin max=grmax ;
    run;
proc univariate data=&data noprint;
    by &KLASS;
    var &VAR;
    output out=_qtile_
           n=n q1=q1  q3=q3  median=median qrange=iqr mean=mean;
data _qtile_;
    set _qtile_;
    By  &KLASS;
    Lo_Notch = Median - 1.58*IQR / sqrt(N);
    Hi_Notch = Median + 1.58*IQR / sqrt(N);
run;
data merged;
    merge &DATA _qtile_;
    by &KLASS;
 /*-----------------------------------------------*
  | Find outside & farout points                  |
  *-----------------------------------------------*/
data plotdat;
    set merged;
    keep &KLASS &VAR &ID outside;
    if &VAR ^= .;
    outside=1;
    if &VAR < (Q1 -1.5*IQR) or &VAR > (Q3 +1.5*IQR)
       then outside=2;
    if &VAR < (Q1 -3.0*IQR) or &VAR > (Q3 +3.0*IQR)
       then outside=3;
run;
data _out_;
   set plotdat;
   if outside > 1 ;
proc sort data=_out_;
   by &KLASS &VAR ;
proc print data=_out_;
   id &ID &KLASS;
   title3 "Outside Observations in Data Set &_DSN_ ";
run;
 /*-----------------------------------------------------*
  |  If connnecting group medians, find them and append |
  *-----------------------------------------------------*/
%if ( &connect ) %then %do;
   data connect;
      set _qtile_(keep=&KLASS Median
                    rename=(Median=&VAR));
      outside=0;
   proc append base=plotdat
               data=connect;
   run;
%end;
 
 /*----------------------------------------------------*
  |  Whiskers go from quartiles to most extreme values |
  |  which are *NOT* outside.                          |
  *----------------------------------------------------*/
data _in_;
   set plotdat;
   if outside = 1;            /* select inside points */
proc univariate data=_in_ noprint;
   by &KLASS;
   var &VAR;                  /* find min and max     */
   output out=_whisk_ min=lo_whisk max=hi_whisk;
run;
data &out;
    merge _qtile_ _whisk_ end=lastobs;
    by &KLASS;
    retain halfmax 1e23 fnmax -1e23;
    drop span halfmax fnmax offset grps;
 
    span = dif ( &KLASS );            /* x(k+1) - x(k) */
    if (_n_ > 1 )
       then halfmax = min( halfmax, span/2);
    fnmax = max( fnmax, &FN );
 
   if ( lastobs ) then do;
      if _n_=1 then halfmax=.5;
      call symput ('HALFMAX', left(put(halfmax,best.)) );
      put ' Maximum possible halfwidth is: ' halfmax /;
      call symput ('FNMAX',  left(put(fnmax,best.)) );
      grps=_n_;
      offset=max(5, 35-5*grps);
      call symput('OFFSET',left(put(offset,2.)) );
      put ' Number of groups: ' grps  'offset=' offset ;
   end;
 
proc print ;
   id &KLASS;
   title3 'BOXPLOT: Quartiles, notches and whisker values';
run;
 /*-----------------------------------------------*
  | Annotate data set to draw boxes & whiskers    |
  *-----------------------------------------------*/
data _dots_;
    set &out;
    retain halfmax &HALFMAX  k ;
    drop k halfmax halfwid hi_notch lo_notch iqr median mean q1  q3 ;
    drop grmin grmax ;
    if ( _n_ = 1) then do;
       set _grsum_;
       K = &WIDTH * HalfMax;
    end;
    halfwid = K * &FN / &FNMax ;
    length function text $8;
    XSYS = '2'; YSYS = '2';
 
   /*    Produce connect-the-dots X, Y pairs */
   X = &KLASS                      ; Y= Lo_Whisk ; dot =  1; link out;
   X = &KLASS                      ; Y= Q1       ; dot =  2; link out;
   X = &KLASS - halfwid            ; Y= Q1       ; dot =  3; link out;
 
%if ( &notch ) %then %do;
   X = &KLASS - halfwid            ; Y= Lo_Notch ; dot =  4; link out;
   X = &KLASS - (1-&F)*halfwid     ; Y= Median   ; dot =  5; link out;
   X = &KLASS - halfwid            ; Y= Hi_Notch ; dot =  6; link out;
%end;
   X = &KLASS - halfwid            ; Y= Q3       ; dot =  7; link out;
   X = &KLASS                      ; Y= Q3       ; dot =  8; link out;
   X = &KLASS                      ; Y= Hi_Whisk ; dot =  9; link out;
   X = &KLASS                      ; Y= Q3       ; dot = 10; link out;
   X = &KLASS + halfwid            ; Y= Q3       ; dot = 11; link out;
%if ( &notch ) %then %do;
   X = &KLASS + halfwid            ; Y= Hi_Notch ; dot = 12; link out;
%end;
   X = &KLASS+(1-&NOTCH*&F)*halfwid; Y= Median   ; dot = 13; link out;
   X = &KLASS-(1-&NOTCH*&F)*halfwid; Y= Median   ; dot = 14; link out;
   X = &KLASS+(1-&NOTCH*&F)*halfwid; Y= Median   ; dot = 15; link out;
%if ( &notch ) %then %do;
   X = &KLASS + halfwid            ; Y= Lo_Notch ; dot = 16; link out;
%end;
   X = &KLASS + halfwid            ; Y= Q1       ; dot = 17; link out;
   X = &KLASS                      ; Y= Q1       ; dot = 18; link out;
 
   X = &KLASS - halfwid/3          ; Y= Lo_Whisk ; dot = 19; link out;
   X = &KLASS - halfwid/3          ; Y= Hi_Whisk ; dot = 19; link out;
   X = &KLASS                      ; Y= Mean     ; dot = 20; link out;
   return;
 
out:
    Select;
       when ( dot=1 ) do;
          FUNCTION = 'MOVE';                   output;
          FUNCTION = 'POLY';                   output;
          End;
       when ( 1< dot <=18) do;
          FUNCTION = 'POLYCONT';               output;
          End;
       when ( dot=19) do;
          FUNCTION = 'MOVE';                   output;
          X = X + 2*halfwid/3 ;
          FUNCTION = 'DRAW';                   output;
          End;
       when ( dot=20) do;
          FUNCTION = 'MOVE';                   output;
          FUNCTION = 'SYMBOL'; TEXT='STAR';    output;
          End;
          Otherwise ;
    End;
    Return;
run;
 
 /*-----------------------------------------------------*
  |  Annotate data set to plot and label outside points |
  *-----------------------------------------------------*/
data _label_;
   set _out_;                          /* contains outliers only */
   by &KLASS;
   keep xsys ysys x y function text style position;
   length text function style $8;
   xsys = '2'; ysys = '2';
   y = &VAR;
   x = &KLASS ;
   function = 'SYMBOL';                     /* draw the point   */
   style = ' ';
   position = ' ';
   if OUTSIDE=2
      then do;  text='DIAMOND'; size=1.7; end;
      else do;  text='SQUARE '; size=2.3; end;
   output;
   %if &ID ^= %str() %then %do;            /* if ID variable,   */
      if first.&KLASS then out=0;
      out+1;
      function = 'LABEL';                  /*  .. then label it */
      text = &ID;
      size=.9;
      style='SIMPLEX';
      x = &KLASS;
      if mod(out,2)=1                      /* on alternating sides*/
         then do; x=x -.05; position='4';  end;
         else do; x=x +.05; position='6';  end;
      output;
   %end;
data _dots_;
   set _dots_ _label_ &anno ;
 /*------------------------------------*
  | Clean up datasets no longer needed |
  *------------------------------------*/
proc datasets nofs nolist library=work memtype=(data);
    delete work _grsum_ merged _in_ _whisk_ _qtile_ _label_;
    run;
options notes;
 /*--------------------------------------*
  | Symbols for connecting group medians |
  *--------------------------------------*/
%if &connect ^= 0  %then %do;
  symbol1 C=BLACK V=NONE I=JOIN L=&connect r=1; /* connected medians   */
  symbol2 C=BLACK V=NONE R=3;                 /* rest done by annotate */
%end;
%else %do;
  symbol1 C=BLACK V=NONE R=3 i=none;          /* all done by annotate  */
%end;
title3;
 
proc gplot data=plotdat ;
  plot &VAR * &KLASS = outside
       / frame nolegend name="&name"
         vaxis=axis1 haxis=axis2 hminor=0
         annotate=_dots_;
  %if %length(&yorder) > 0 %then
     %let yorder = order=(&yorder);
  axis1
       &yorder
       value=(h=1.2) label =(&yangle h=1.5);
  axis2 value=(h=1.2) label =(h=1.5) offset=(&offset pct);
  %if &varfmt ^=  %str() %then %do; format &var   &varfmt ;      %end;
  %if &classfmt^= %str() %then %do; format &KLASS &classfmt ;    %end;
  %if &varlab ^=  %str() %then %do; label  &var   = "&varlab";   %end;
  %if &classlab^= %str() %then %do; label  &KLASS = "&classlab"; %end;
  run;
%mend boxplot;
 
 /*----------------------------------*
  | Count number of &CLASS variables |
  *----------------------------------*/
%macro nvar(varlist);
   %local wvar result;
   %let result = 1;
   %let wvar = %nrbquote(%scan( &varlist, &result));
   %do %until ( &wvar= );
       %let result = %eval( &result + 1);
       %let wvar = %nrbquote(%scan( &varlist, &result));
   %end;
   %eval( &result - 1)
%mend nvar;
