/*-------------------------------------------------------------------*
 * CANPLOT SAS    Canonical discriminant structure plot.             *
 *         Plots class means on two canonical variables, confidence  *
 *         circles for those means, and variable vectors showing the *
 *         correlations of variables with the canonical variates.    *
 *-------------------------------------------------------------------*
 *  Author:  Michael Friendly            <FRIENDLY@YORKVM1>          *
 * Created:  24 Nov 1991 13:17:10                                    *
 * Revised:  26 Nov 1991 08:38:35                                    *
 * Version:  1.0                                                     *
 *                                                                   *
 *-------------------------------------------------------------------*/
/*               Copyright (c) 1994, Michael Friendly.               * 
 * Permission to use and distribute this program is granted as long  *
 * as this copyright notice remains intact.  Please acknowledge      *
 * Michael Friendly as author of this program in publications which  *
 * result from its use.                                              * 
 *-------------------------------------------------------------------*/
 
%macro canplot(
     data=_last_,  /* name of data set to analyze               */
     class=,       /* name of class variable                    */
     var=,         /* list of classification variables          */
     scale=4,      /* scale factor for variable vectors in plot */
     conf=.99,     /* confidence probability for canonical means*/
     out=_dscore_, /* output data set containing discrim scores */
     anno=_danno_, /* output data set containing annotations    */
     plot=YES,     /* or NO to suppress plot                    */
     haxis=,       /* AXIS statement for horizontal axis        */
     vaxis=,       /* and for vertical axis- use to equate axes */
     name=CANPLOT, /* name for graphic catalog entry            */
     colors=RED GREEN BLUE BLACK PURPLE YELLOW BROWN ORANGE,
     symbols=+ square star -     plus   :      $     = );
 
%let plot = %upcase(&PLOT);
%if %length(&var)=0 %then %do;
    %put CANPLOT:  You must specify VAR= variables list ;
    %goto DONE;
    %end;
%if %length(&class)=0 or %length(%scan(&class,2))>0 %then %do;
    %put CANPLOT:  You must specify one CLASS= variable ;
    %goto DONE;
    %end;
 
proc candisc data=&data
             out=&out short
             outstat=_dstat_;
   classes &class;
   var &var ;
proc sort data=&out;
   by &class;
proc means noprint;
   var can1 can2;
   by &class;
   output out=_means_ mean=can1 can2 n=n;
 
data &anno;
   set _means_ end=eof;
   length text color function $8;
   retain xsys '2' ysys '2';
   drop can1 can2 n;
   x = can1;
   y = can2;
   color=scan("&colors",_n_);
   /* mark the class mean        */
   text = 'PLUS';
   hsys = '4'; size = 4;
   function = 'SYMBOL';      output;
 
   /* draw confidence region */
   size = sqrt( cinv(&conf, 2, 0) / n ) ;   * radius ;
   line = 20;
   do a = 0 to 360 by 10;                   * draw a "circle" ;
      ang = a*arcos(-1)/180;                * convert to radians;
      xp=  size * cos(ang);
      yp=  size * sin(ang);
      x = xp+ can1;                         * translate to means;
      y = yp+ can2;
      if a=0 then FUNCTION = 'MOVE    ';
             else FUNCTION = 'DRAW    ';
      output;
   end;
   if eof then do;          * save number of groups ;
      call symput('NGP',put(_n_,best5.));
      end;
data _coeff_;
   set _dstat_;             * get standardized coefficients ;
   drop _TYPE_ ;
   if _type_ = 'STRUCTUR' |  /* Version 6 */
      _type_ = 'TSTRUCT'  ;  /* Version 5 */
proc transpose out=_coeff_;
proc print data=_coeff_;
 
data _vector_;
   set _coeff_;
   where (can1^=.);
   length function text $8;
   retain xsys '2' ysys '2' position '6';
   x = 0 ; y = 0;
   function = 'MOVE ' ; output;
   x = &scale * can1 ;
   y = &scale * can2 ;
   function = 'DRAW ' ; output;
   text = _NAME_;
   function = 'LABEL' ; output;
data &anno;
   set &anno _vector_;
 
%if &plot = YES %then %do;
   %gensym(n=&ngp, h=1.1, symbols=&symbols, colors=&colors);
   %if %length(&haxis)=0 %then %do;
      axis2 label=(h=1.3) value=(h=1.2);
      %let haxis=axis2;
      %end;
   %if %length(&vaxis)=0 %then %do;
      axis1 label=(h=1.3) value=(h=1.2);
      %let vaxis=axis1;
      %end;
   proc gplot data=&out ;
      plot can2 * can1 = &class
           / anno=&anno frame
             vaxis=&vaxis haxis=&haxis legend=legend1
             hm=1 vm=1 name="&name" ;
      legend1 label=(h=1.2) value=(h=1.2);
      run; quit;
   %end;
proc datasets lib=work memtype=data nolist;
   delete _coeff_ _dstat_ _means_ _vector_;
%done:
%mend;
 
 /*----------------------------------------------------*
  |  Macro to generate SYMBOL statement for each GROUP |
  *----------------------------------------------------*/
%macro gensym(n=1, h=1.5, i=none,
              symbols=%str(- + : $ = X _ Y),
              colors=BLACK RED GREEN BLUE BROWN YELLOW ORANGE PURPLE);
    %*-- note: only 8 symbols & colors are defined;
    %*--    revise if more than 8 groups (recycle);
  %local chr col k;
  %do k=1 %to &n ;
     %let chr =%scan(&symbols, &k,' ');
     %let col =%scan(&colors, &k, ' ');
     symbol&k h=&h v=&chr c=&col i=&i;
  %end;
%mend gensym;
