 /*-------------------------------------------------------------------*
  * CORRESP SAS     Correspondence analysis of contingency tables     *
  *                                                                   *
  *-------------------------------------------------------------------*
  *  Author:  Michael Friendly            <FRIENDLY@YORKVM1>          *
  * Created:  19 Jan 1990 15:23:09                                    *
  * Revised:  21 Oct 1991 09:46:50                                    *
  * Version:  1.1                                                     *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *                                                                   *
  *-------------------------------------------------------------------*/
%macro CORRESP(
     data=_LAST_,        /* Name of input data set               */
     var=,               /* Column variables                     */
     id=,                /* ID variable: row labels              */
     out=COORD,          /* output data set for coordinates      */
     anno=LABEL,         /* name of annotate data set for labels */
     rowht=1,            /* height for row labels                */
     colht=1             /* height for col labels                */
     );
 /*------------------------------------------*
  |  IML routine for Correspondence Analysis |
  *------------------------------------------*/
Proc IML;
Start CORRESP(F,RowId,Vars);
   I = nrow(F);
   J = ncol(F);
   R = F[,+];                   * Row totals;
   C = F[+,];                   * Col totals;
   N = F[+];                    * Grand total;
   E  = R * C / N;              * Expected frequencies;
   D = (F - E) / sqrt(E);       * Standardized deviates;
   D = D / sqrt( N );
   DPD = D` * D;
   Inertia = trace(DPD);
   Chisq = N * Inertia;         * Total chi-square;
   DF = (I-1)*(J-1);
   reset noname;
   C1={'ChiSq'};
   C2={'DF'};
   Print 'Overall Association', CHISQ[colname=C1]
                                DF[colname=C2 format=6.0];
 
   call eigen(values, vectors, dpd);
   k = min(I,J)-1;                * number of non-zero eigenvalues;
   values  = values[1:k];
   cancorr = sqrt(values);        * singular values = Can R;
   chisq = n * values ;           * contribution to chi-square;
   percent = 100* values / inertia;
      *-- Cumulate by multiplying by lower triangular matrix of 1s;
   tri= (1:k)`*repeat(1,1,k)  >= repeat(1,k,1)*(1:k) ;
   cum = tri*percent;
   C1={'Singular Values'};
   C2={'Inertias'};
   C3={'Chi-Squares'};
   C4={'Percent'};
   C5={' Cum % '};
   print 'Singular values, Inertia, and Chi-Square Decomposition',,
         cancorr [colname=C1 format=9.4]
         values  [colname=C2 format=9.4]
         chisq   [colname=C3 format=9.3]
         percent [colname=C4 format=8.2]
         cum     [colname=C5 format=8.2];
 
   L = values[1:2];
   U = vectors[,1:2];
   Y = diag(1/sqrt(C/N)) * U * diag(sqrt(L));
   X = diag(N/R) * (F / N) * Y * diag(sqrt(1/L));
   C1={'DIM1' 'DIM2'};
   Print 'Row Coordinates'   , X [Rowname=RowId Colname=C1];
   Print 'Column Coordinates', Y [Rowname=Vars  Colname=C1];
 
   OUT = X // Y;
   ID  = RowId // Vars`;
*  Call PGRAF(OUT,ID,'Dimension 1', 'Dimension 2', 'Row/Col Association');
   TYPE = repeat({"OBS "},I,1) // repeat({"VAR "},J,1);
   ID  = concat(TYPE, ID);
   Create &out  from OUT[rowname=ID colname={"Dim1" "Dim2"}];
   Append from OUT[rowname=ID];
Finish;
 
Use &data;
Read all VAR {&var} into F [Rowname=&id Colname=Vars];
 
Run CORRESP(F,&id,Vars);
quit;
 /*----------------------------------*
  |  Split ID into _TYPE_ and _NAME_ |
  *----------------------------------*/
data &out;
   set &out;
   drop id;
   length _type_ $3 _name_ $16;
   _type_ = substr(id,1,3);
   _name_ = substr(id,5);
proc print data=&out;
   id _type_ _name_;
 /*--------------------------------------------------*
  | Annotate row and column labels                   |
  *--------------------------------------------------*/
data &anno;
   set &out;
   length function $8 text $16;
   xsys='2'; ysys='2';
   text = _name_;
   style='DUPLEX';
   x = dim1; y = dim2;
 
   if _type_ = 'OBS' then do;
      size= &rowht ;
      color='BLACK';
      position='5';
      function='LABEL   '; output;
      end;
 
   if _type_ = 'VAR' then do;
      color='RED  ';
      size= &colht;
      if dim1 >=0
         then position='6';             /* left justify       */
         else position='4';             /* right justify      */
      function='LABEL   '; output;
      end;
%mend CORRESP;
