 /*-------------------------------------------------------------------*
  * DENSITY SAS    Nonparametric density estimates from a sample.     *
  *                                                                   *
  * User chooses a bandwidth parameter to balance smoothness and bias *
  * and the range of the data over which the density is to be fit.    *
  *-------------------------------------------------------------------*
  *  Author:  Michael Friendly            <FRIENDLY@YORKVM1>          *
  * Created:  23 Mar 1989 16:21:12                                    *
  * Revised:  11 Jun 1991 12:05:09                                    *
  * Version:  1.0                                                     *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *                                                                   *
  *-------------------------------------------------------------------*
  *      Original program by: C. ROGER LONGBOTHAM                     *
  *      while at Rockwell International, Rocky Flats Plant           *
  * From: SAS SUGI 12, 1987, 907-909.                                 *
  *-------------------------------------------------------------------*/
 
%macro DENSITY(
     data=_LAST_,        /* Name of input data set              */
     out=DENSPLOT,       /* Name of output data set             */
     var=X,              /* Input variable (numeric)            */
     window=,            /* Bandwidth (H)                       */
     xfirst=.,           /* . or any real; smallest X value     */
     xlast=.,            /* . or any real; largest X value      */
     xinc=. );           /* . or value>0; X-value increment     */
                         /* Default: (XLAST-XFIRST)/60          */
data _in_;
   set &data;
   keep &var;
   if &var ^= .;
proc sort data=_in_;
   by &var;
 
proc iml;
 
start WINDOW;    *-- Calculate default window width;
   mean = xa[+,]/n;
   css = ssq(xa - mean);
   stddev = sqrt(css/(n-1));
   q1 = floor(((n+3)/4) || ((n+6)/4));
   q1 = (xa[q1,]) [+,]/2;
   q3 = ceil(((3*n+1)/4) || ((3*n-2)/4));
   q3 = (xa[q3,]) [+,]/2;
   quartsig = (q3 - q1)/1.349;
   h  = .9*min(stddev,quartsig) * n##(-.2);  * Silvermans formula;
   finish;
 
start INITIAL;   *-- Translate parameter options;
   if xf=. then xf=xa[1,1];
   if xl=. then xl=xa[n,1];
   if xl <= xf then do;
      print 'Either largest X value chosen is too small';
      print 'or all data values are the same';
      stop;
      end;
   if dx=. | dx <= 0 then do;
      inc = (xl-xf)/60;
      rinc = 10 ## (floor(log10(inc))-1);
      dx = round(inc,rinc);
      end;
   if xf=xa[1,1] then xf=xf-dx;
   nx = int((xl-xf)/dx) +  3;
   finish;
 
*-- calculate density at specified x values;
start DENSITY;
   fnx = j(nx,3,0);
   vars = {"DENSITY"  "&VAR" "WINDOW"};
   create &out from fnx [colname=vars];
   sigmasqr = .32653;                   * scale constant for kernel  ;
   gconst = sqrt(2*3.14159*sigmasqr);
   nuh = n*h;
   x = xf - dx;
   do i = 1 to nx;
      x = x + dx;
      y = (j(n,1,x) - xa)/h;
      ky = exp(-.5*y#y / sigmasqr) / gconst;      * Gaussian kernel;
      fnx[i,1] = sum(ky)/(nuh);
      fnx[i,2] = x;
      end;
   fnx[,3] = round(h,.001);
   append from fnx;
   finish;
 
*-- Main routine ;
   use _in_;
   read all var "&var" into xa [colname=invar];
   n = nrow(xa);
   %if &window=%str() %then %do;
      run window;
      %end;
   %else %do;
      h = &window ;
      %end;
 
   xf    = &xfirst;
   xl    = &xlast;
   dx    = &xinc;
   run initial;
   run density;
   close &out;
   quit;
%mend DENSITY;
