*------------------ LINEARLY OPTIMAL PROFILES --------------------*
        A scaling procedure for profiles data which represents
the profiles of  N  cases on   P variables by a set of linear
profiles.
 
Reference: Hartigan, Clustering Algorithms, Wiley, 1975, 34-38.
*------------------------------------------------------------------;
/*               Copyright (c) 1994, Michael Friendly.               * 
 * Permission to use and distribute this program is granted as long  *
 * as this copyright notice remains intact.  Please acknowledge      *
 * Michael Friendly as author of this program in publications which  *
 * result from its use.                                              * 
 *-------------------------------------------------------------------*/

%macro linpro(data=_last_, 
	var=_numeric_,
	id=, 
	std=STD,
	plot=YES, 
	out=linplot,
	fit=linfit,
	anno=vlabel,
	name=linpro);

proc iml;
  start linpro;
   *--- Get input matrix and dimensions;
   use &data;
   read all var{&var} into x[ rowname=&id  colname=vars ];
   n = nrow(x);
   p = ncol(x);
 
   *--- Compute covariance matrix of x;
   m = x[ : ,];                   *- means;
   d = x - j( n , 1) * m;         *- deviations from means;
   c = d` * d /  n;               *- variance-covariance matrix;
   sd = sqrt( vecdiag( c))`;      *- standard deviations;
 
   *--- Prestandardize if necessary;
   ratio = sd[<>] / sd[><];
   if ratio > 2.5 then do;
      s = sd;
      c = diag( 1 / sd) * c * diag( 1 / sd);
      print "Analyzing the correlation matrix";
   end;
   else do;
      s = j( 1 , p);
      print "Analyzing the covariance matrix";
   end;
   print c [ colname=vars rowname=vars f=9.2];
 
   *--- Eigenvalues & vectors of C;
   call eigen ( val , vec , c );
   tr = val[+];
   pc = val / tr ;
   cum = pc[1];
   do i = 2 to p;
      cum = cum // ( cum[ i-1 ] + pc[ i ]);
   end;
   val = round(val,.001) || round((100 * pc),.01) || round((100 * cum ),.01) ;
   cl = { 'EIGVALUE' '%_TRACE' 'CUM_%'};
   print "Eigenvalues and Variance Accounted for"
         , val [ colname=cl ];
 
   *--- Scale values;
   e1 = vec[ , 1] # sign( vec[<> , 1]);
   e2 = vec[ , 2];
   d = d / ( j( n , 1) * ( s # e1` ) );
   dat = d / ( j( n , 1) * ( s  ) );
   pos = e2 / e1;
 
   *--- For case i, fitted line is  Y = F1(I) + F2(I) * X   ;
   f1 = ( d * e1 ) / e1[## ,];
   f2 = ( d * e2 ) / e2[## ,];
   f = f1 || f2;
 
   *--- Output the results;
   scfac = round( ( sd` # e1 ) , .1);
   table = ( m` ) || ( sd` ) || e1 || e2 || scfac || round(pos,.01);
   ct = { 'MEAN' 'STD_DEV' 'EIGV1' 'EIGV2' 'SCALE' 'POSITION'};
   print , table [ rowname=vars colname=ct ];
 
   *--- Fitted values, residuals;
   fit = f1 * j( 1 , p) + f2 * pos`;
   resid = d - fit;
   print "D" , d [ rowname=&id colname=vars format=8.3 ];
   print "dat" , dat [ rowname=&id colname=vars format=8.3 ];
   print "Fitted values" , fit [ rowname=&id colname=vars format=8.3 ];
   print "Residuals"     , resid [ rowname=&id colname=vars format=8.3 ];

	mf = fit[:,];
	vf = sqrt(fit[##,] /n);
	mr = resid[:,];
	vr = sqrt(resid[##,] /n);
	ratio = vf / sd;
	
*	print 'Mean fit' , mf[colname=vars format=8.3];
	print 'SD  fit' , vf[colname=vars format=8.3]; 
*	print 'Mean res' , mr[colname=vars format=8.3]; 
	print 'SD  res' , vr[colname=vars format=8.3];
	print 'SD Ratio Fit/Dat', ratio[colname=vars format=8.3]; 

	ratio = fit / d;
	print 'Ratio Fit/Dat', ratio[colname=vars format=8.3]; 
	mr = ratio[:,];
	print 'Mean Ratio Fit/Dat', mr[colname=vars format=8.3]; 
		 
   *--- Rearrange columns;
   s = rank( pos);
   zz = table;  table [ s , ] = zz;
   zz = vars;   vars [ , s ] = zz;
   zz = pos;    pos [ s , ] = zz;
   zz = scfac;  scfac [ s , ] = zz;
   zz = d;      d [ , s ] = zz;
   print "TABLE, with variables reordered by position"
         , table [ rowname=vars colname=ct ];
   lt = { 'INTERCPT' 'SLOPE'};
   print "Case lines"    , f [ rowname=&id colname=lt f=9.4 ];
 
   *--- Construct output array -
          residuals bordered by fitted scale values;
   v1 = val[ 1 ] || {0};
   v2 = {0} || val[ 2 ];
   xout = ( resid || f ) //
           ( pos` || v1 ) //
         ( scfac` || v2 );
   rl = { 'VARIABLE','SCALE'};
   rl = shape( &id ,0 , 1) // rl;
   cl = { 'INTERCPT','SLOPE'};
   cl = shape( vars , p) // cl;
   create &fit from xout[ rowname=rl colname=cl ];
   append from xout[ rowname= rl ];
   free rl cl xout;
 
   *--- Output the array to be plotted;
   do col = 1 to p;
       rows = j(n,1,pos[col,]) ||  fit[,col] || resid[,col];
       pout = pout // rows;
       rl = rl // shape(&id,1);
       end;
   cl = { 'POSITION' 'FIT' 'RESIDUAL'};
   create &out from pout[ rowname=rl colname=cl ];
   append from pout[ rowname=rl ];
 finish;
 run linpro;
quit;

data &out;
  set &out(rename=rl=&id);

proc print data=&fit;
   id rl;
*   title3 'R E S I D U A L S, bordered by fitted coefficients';
*   title4 '(Variables sorted by position)';
   format _numeric_ 8.2;
/* find max x position */

data _null_;
	set &fit;
	where(rl='VARIABLE');
	last = max(of &var);
	call symput('xmax', put(last,best6.));
	run;

data vlabel;
   set &fit;
   length text $12 function $8;
   array vars &var;
   Select;
      when (RL='VARIABLE') do;
         xsys='2'; ysys='1';
         angle = 45;
         position='6';
         do over vars;
            x = vars;
            y = 0; function='MOVE';  output;
            y = 1; function='DRAW';  output;
            call vname(vars,text);
            y = 2; function='LABEL'; output;
            end;
      end;
      when (RL='SCALE') ;        /* nothing */
      otherwise do;              /* observations */
         xsys='2'; ysys='2';
         angle = 0;
         position='C';
         function='LABEL';
         text = RL;              /* name of obs */
         x = &xmax;              /* data dependent: largest position */
         y = INTERCPT + SLOPE * x;
         x = x + .1;
         output;
      end;
   end; /* select */

proc gplot data=&out &GOUT ;
   plot fit * position = &id
        / vaxis=axis1 haxis=axis2
          anno=vlabel nolegend
          name="&name" ;
   symbol1 i=join v=- h=.5 l=1 c=black r=4;
   symbol2 i=join v=- h=.5 l=3 c=red   r=4;
   symbol3 i=join v=- h=.5 l=5 c=blue  r=4;
   symbol4 i=join v=- h=.5 l=9 c=green r=4;
   symbol5 i=join v=- h=.5 l=11 c=orange r=4;
   axis1 label=(a=90 r=0);
   axis2 minor=none style=0 offset=(,5);
   label fit ='Fitted Value';

%mend;
