 /*-------------------------------------------------------------------*
  * LOWESS SAS     Locally weighted robust scatterplot smoothing      *
  *                                                                   *
  *-------------------------------------------------------------------*
  *  Author:  Michael Friendly            <FRIENDLY@YORKVM1>          *
  * Created:  21 Apr 1989 09:21:55                                    *
  * Revised:  4 Sep 1991 11:30:24                                     *
  * Version:  1.3                                                     *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *                                                                   *
  *-------------------------------------------------------------------*/
%macro LOWESS(
       data=_LAST_,        /* name of input data set            */
       out=SMOOTH,         /* name of output data set           */
       x = X,              /* name of independent variable      */
       y = Y,              /* name of Y variable to be smoothed */
       id=,                /* optional row ID variable          */
       f = .50,            /* lowess window width               */
       iter=2,             /* total number of iterations        */
       plot=NO,            /* draw the plot?                    */
       name=LOWESS);       /* name for graphic catalog entry    */
options nonotes;
proc sort data=&data;
   by &x;
 
proc iml;
start WLS( X, Y, W, B, I );      *-- Weighted least squares;
   x = j(nrow(x), 1, 1) || x;
   xpx = x` * diag( w ) * x;
   xpy = x` * diag( w ) * y;
   if abs(det(xpx)) > .00001
      then b   = inv(xpx) * xpy;
      else do;
         b = (y[loc(w^=0)])[:] // { 0 } ;
         print 'Singular matrix for observation', I;
      end;
finish;
 
start MEDIAN( W, M);         * calculate median ;
   n = nrow( W );
   R = rank( W );
   i = int((n+1)/2);
   i =  i || n-i+1;
   M = W[ R[i] ];
   M = .5 # M[+];
finish;
 
start ROBUST( R, WTS);       * calculate robustness weights;
    run median(abs(R), M);
    W = R / (6 # M);         * bisquare function;
    WTS = (abs(W) < 1) # (1 - W##2) ## 2;
finish;
 
start LOWESS( X, Y, F, STEPS, YHAT, RES, DELTA);
   n = nrow(X);
   if n < 2 then do;
      yhat = y;
      return;
      end;
   q = round( f * n);      * # nearest neighbors;
 
   res  = y;
   yhat = J(n,1,0);
   delta= J(n,1,1);        * robustness weights;
   if steps <= 0 then steps=1;
   do it = 1 to steps;
      do i = 1 to n;
         dist = abs( x - x[i] );      * distance to each other pt;
         r = rank( dist );
         s = r; s[r]=1:n;
         near =  s[1:q] ;             * find the q nearest;
         nx = x [ near ];
         ny = y [ near ];
         d  = dist[ near[q] ];        * distance to q-th nearest;
         if d > 0 then do;
            u = abs( nx - x[i] ) / d ;
            wts = (u < 1) # (1 - u##3) ## 3; * neighborhood wts;
            wts = delta[ near ] # wts;
            if sum(wts[2:q]) > .0001 then do;
               run wls( nx, ny, wts, b, i );
               yhat[i] = (1 || x[i]) * b;      * smoothed value;
               end;
            else yhat[i] = y[i];
         end;
         else do;
            yhat[i] = ny [+] /q;
         end;
      end;
      res = y - yhat;
      run robust(res,delta);
   end;
finish;
 
*-- Main routine;
  use &data;
  %if &id.NULL=NULL %then %let rowid=;
  %else %let rowid=rowname=&id;
  read all var{&x &y} into xy[ colname=vars &rowid ];
  close &data;
  x = xy[,1];
  y = xy[,2];
  run lowess(x, y, &f, &iter, yhat, res, weight);
  xyres =x || y || yhat || res || weight;
  cname = vars || {"_YHAT_" "_RESID_" "_WEIGHT_" };
  print "Data, smoothed fit, residuals and weights",
        xyres[ colname=cname &rowid ];
 
*-- Output results to data set &out ;
  xys =    yhat || res || weight;
  cname = {"_YHAT_" "_RESID_" "_WEIGHT_" };
 
  create &out from xys [ colname=cname ];
     append from xys;
quit;
 /*--------------------------------------------*
  | Merge data with smoothed results.          |
  | (In a data step to retain variable labels) |
  *--------------------------------------------*/
data &out;
   merge &data(keep=&x &y &id)
         &out ;
   label _yhat_ = "Smoothed &y"
         _weight_='Lowess weight';
%if %upcase(&PLOT)=YES %then %do;
proc gplot data=&out &GOUT ;
  plot &y     * &x = 1
       _yhat_ * &x = 2
       / overlay frame
         vaxis=axis1 haxis=axis2
         name="&name" ;
  symbol1 v=+ h=1.5  i=none c=black;
  symbol2 v=none i=join c=red;
  axis1   label=(h=1.5 f=duplex a=90 r=0) value=(h=1.3);
  axis2   label=(h=1.5 f=duplex)          value=(h=1.3);
  run;
options notes;
%end;
%mend LOWESS;
