 /*-------------------------------------------------------------------*
  * OUTLIER SAS    Robust multivariate outlier detection              *
  *                                                                   *
  * Macro to calculate robust Mahalanobis distances for each          *
  * observation in a dataset. The results are robust in that          *
  * potential outliers do not contribute to the distance of any       *
  * other observations.                                               *
  *                                                                   *
  * The macro makes one or more passes through the data. Each         *
  * pass assigns 0 weight to observations whose DSQ value             *
  * has prob < PVALUE. The number of passes should be determined      *
  * empirically so that no new observations are trimmed on the        *
  * last pass.                                                        *
  *-------------------------------------------------------------------*
  *  Author:  Michael Friendly            <FRIENDLY@YORKVM1>          *
  * Created:  16 Jan 1989 18:38:18                                    *
  * Revised:  11 Jun 1991 12:16:31                                    *
  * Version:  1.0                                                     *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *                                                                   *
  *-------------------------------------------------------------------*/
%macro OUTLIER(
         data=_LAST_,      /* Data set to analyze            */
         var=_NUMERIC_,    /* input variables                */
         id=,              /* ID variable for observations   */
         out=CHIPLOT,      /* Output dataset for plotting    */
         pvalue=.1,        /* Prob < pvalue -> weight=0      */
         passes=2,         /* Number of passes               */
         print=YES);       /* Print OUT= data set?           */
 
 /*-------------------------------------------------------*
  | Add WEIGHT variable. Determine number of observations |
  | and variables, and create macro variables.            |
  *-------------------------------------------------------*/
data in;
   set &data end=lastobs;
   array invar{*} &var;
   _weight_ = 1;               /* Add weight variable */
   if ( lastobs ) then do;
      call symput('NOBS', _n_);
      call symput('NVAR', left(put(dim(invar),3.)) );
      end;
 
%do pass = 1 %to &PASSES;
   %if &pass=1 %then %let in=in;
               %else %let in=trimmed;
 /*--------------------------------------------------------------*
  | Transform variables to scores on principal components.       |
  | Observations with _WEIGHT_=0 are not used in the calculation,|
  | but get component scores based on the remaining observations.|
  *--------------------------------------------------------------*/
   proc princomp std noprint data=&in out=prin;
      var &var;
      freq _weight_;
 
 /*-----------------------------------------------------------*
  | Calculate Mahalanobis D**2 and its probability value. For |
  | standardized principal components, D**2 is just the sum   |
  | of squares. Output potential outliers to separate dataset.|
  *-----------------------------------------------------------*/
   data out1    (keep=pass _case_ &id dsq prob)
        trimmed (drop=pass _case_ );
      set prin ;
      pass = &pass;
      _case_ = _n_;
 
      dsq = uss(of prin1-prin&nvar);    /* Mahalanobis D**2 */
      prob = 1 - probchi(dsq, &nvar);
      _weight_ = (prob > &pvalue);
      output trimmed;
      if _weight_ = 0 then do;
         output out1   ;
         end;
   run;
   proc append base=outlier data=out1;
%end;
   proc print data=outlier;
   title2 'Observations trimmed in calculating Mahalanobis distance';
 /*------------------------------------------*
  | Prepare for Chi-Square probability plot. |
  *------------------------------------------*/
proc sort data=trimmed;
   by dsq;
data &out;
   set trimmed;
   drop prin1 - prin&nvar;
   _weight_ = prob > &pvalue;
   expected = 2 * gaminv(_n_/(&nobs+1), (&nvar/2));
 
%if &print=yes %then %do;
proc print data=&out;
   %if &id ^=%str() %then
   %str(id &id;);
   title2 'Possible multivariate outliers have _WEIGHT_=0';
%end;
 
%if &ID = %str() %then %let SYMBOL='*';
                 %else %let SYMBOL=&ID;
proc plot data=&out;
   plot dsq      * expected = &symbol
        expected * expected = '.'   /overlay hzero vzero;
title2 'Chi-Squared probability plot for multivariate outliers';
run;
%done:
proc datasets nofs nolist;
   delete outlier out1;
%mend outlier;
