 /*-------------------------------------------------------------------*
  * PARTIAL SAS - IML macro program for partial regression residual   *
  *               plots.                                              *
  *-------------------------------------------------------------------*
  *  Author:  Michael Friendly            <friendly@VM1.YorkU.ca>     *
  * Created:  23 Jan 1989 16:11:15                                    *
  * Revised:  1 Oct 1991 08:42:12                                     *
  * Version:  1.3                                                     *
  *      From ``The SAS System for Statistical Graphics''             *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *                                                                   *
  *-------------------------------------------------------------------*/
%macro PARTIAL(
        data = _LAST_,  /* name of input data set               */
        yvar =,         /* name of dependent variable           */
        xvar =,         /* list of independent variables        */
        id =,           /* ID variable                          */
        label=INFL,     /* label ALL, NONE, or INFLuential obs  */
        out =,          /* output data set: partial residuals   */
		  htext=1.5,      /* height of text in plots              */
		  plots=&xvar,    /* which partial plots to produce       */
        gout=gseg,      /* name of graphic catalog              */
        name=PARTIAL);  /* name of graphic catalog entries      */
 
%let label = %UPCASE(&label);
Proc IML;
start axes (xa, ya, origin, len, labels);
    * arguments:
        xa, ya : values to be plotted, world coords
        origin : screen coordinates (x,y) of plot origin
        len    : axis lengths, screen coords
        labels : x,y axis labels ;
   col= 'black';
   ox = origin[1];
   oy = origin[2];
   %if &sysver < 6 %then %do;     /* Version 5 */
      call gxaxis(origin,len,xa,7) color=col format='7.1';
      call gyaxis(origin,len,ya,7) color=col format='7.1';
      %end;
   %else %do;                     /* Version 6 */
      call gset('height', &htext);
      call gset('font','duplex');
      /*---define viewport and window for the application---*/
      call gport (ox, oy, ox+len, oy + len);
      call gscale(sx, xa, 6);
      call gscale(sy, ya, 6);
      nx =1 + (sx[2]-sx[1])/sx[3];
      ny =1 + (sy[2]-sy[1])/sy[3];
      wd = sx[1:2] || sy[1:2];
      call gwindow (wd);
      if all(mod(wd,1)=0) then fmt='6.0';
                          else fmt='7.1';
      xl = wd[2,1]-wd[1,1];    *--length of x axis (world);
      yl = wd[2,2]-wd[1,2];    *--length of y axis;
      call gxaxis( wd[1,], xl,nx,,,fmt,,,col);
      call gyaxis( wd[1,], yl,ny,,,fmt,,,col);
      /*--restore to default window and viewport for axes labels--*/
      call gport(0,0,100,100);
      call gwindow(0,0,100,100);
      %end;
   xo = ox + len/2 - length(labels[1])/2;
   yo = oy - 8;
   call gscript(xo,yo,labels[1]) color=col;
   xo = ox - 9;
   yo = oy + len;
   if nrow(labels)>1 | ncol(labels)>1
      then call gscript(xo,yo,labels[2]) angle=270 rotate=90 color=col;
   %if &sysver >= 6 %then %do;
      /*--redefine window and viewport for the plot--*/
      call gport (ox, oy, ox+len, oy+len);
      call gwindow (wd);
      %end;
finish;
 
*-----Find partial residuals for each variable-----;
start partial(x, y, names, obs, uv, uvname );
   k = ncol(x);
   n = nrow(x);
   yname = names[,k+1];
   k1= k + 1;                  *-- number of predictors;
   x = j( n , 1 , 1) || x;     *-- add column of 1s;
   name1 = { 'INTCEPT'};
   names = name1 || names[,1:k];
 
   run reg( y,  x, b, res, yhat, hat, rstudent );
   print "Full regression";
   print "Regression weights" , b[ rowname=names ];
   lev = hat > 2*k1/n;
   flag = lev | (abs(rstudent) > 2);
   if any( lev ) then do;
      l = loc(lev)`;
		ol = obs[l];
      xl=  x[l,2:k1];
      Print "High leverage or large residual points",
				 XL [ r=ol c=(names[2:k1]) ];
   end;
 
   do i = 1 to k1;
      name = names[,i];
      reset noname;
      free others;
      do j = 1 to k1;
         if j ^=i then others = others || j;
      end;
      run reg( y,    x[, others], by, ry, fy, hy, sry );
      run reg( x[,i],x[, others], bx, rx, fx, hx, srx );
      uv = uv || ry ||rx;
      uvname = uvname || concat({'U'},name)
                      || concat({'V'},name);
 
      if i>1 then do;	*-- Skip plot for intercept;
		if index("&plots",trim(name)) > 0 then print "plotting" name;
       /**--------------------------------**
        | Start IML graphics               |
        **--------------------------------**/
       %if &sysver < 6 %then %do;
          %let lib=%scan(&gout,1,'.');
          %let cat=%scan(&gout,2,'.');
          %if &cat=%str() %then %do;
              %let cat=&lib;
              %let lib=work;
              %end;
          call gstart gout={&lib &cat}
                name="&name"
                descript="Partial regression plot for &data";
       %end;
       %else %do;     /* Version 6 */
          call gstart("&gout");
          call gopen("&name",1,"Partial regression plot for &data");
       %end;
 
         labels = concat( {"Partial "}, name )  ||
                  concat( {"Partial "}, yname ) ;
         run axes(rx,ry,{15 15}, 75, labels) ;
         call gpoint(rx,ry,,'black');
 
         *-- Draw regression line from slope;
         xs = rx[<>] // rx[><];
         ys = b[i] * xs;
         call gdraw(xs, ys, 3, 'BLUE');
 
         *-- Mark influential points and large residuals;
         %if &label ^= NONE %then %do;
            outy = ry[ loc(flag) ];
            outx = rx[ loc(flag) ];
            outl = obs[ loc(flag) ];
            call gpoint(outx, outy,,'RED');
            call gtext(outx,outy,outl,'RED');
         %end;
         %if &label = ALL %then %do;
            outy = ry[ loc(^flag) ];
            outx = rx[ loc(^flag) ];
            outl = obs[ loc(^flag) ];
            call gtext(outx,outy,outl,'black');
         %end;
         call gshow;
      end;
   end;
   print "Partial Residuals", uv[ colname=uvname format=8.3];
finish;  /* end of partial */
 
*----- module to fit one regression ----------;
start reg (y, x, b, res, yhat, h, rstudent);
	n = nrow(x);
	p = ncol(x);
	xpx = x` * x;
	xpy = x` * y;
	xpxi= inv(xpx);
	b   = xpxi * xpy;
	yhat= x * b;
	res = y - yhat;
	h   = vecdiag(x * xpxi * x`);
	sse = ssq(res);
	sisq= j(n,1,sse) - (res##2) / (1-h);
	sisq= sisq / (n-p-1);
	rstudent = res / sqrt( sisq # (1-h) );
finish;
 
*-----read the data and prepare partial regression plots----;
     use &data;
     %if &id ^= %str() %then %do;
        read all var{&xvar} into  x[ rowname=&id colname=xname ];
     %end;
     %else %do;
        read all var{&xvar} into  x[ colname=xname ];
        %let id = obs;
        obs = char(1:nrow(x),3,0);
     %end;
     read all var{&yvar } into  y[ colname=yname ];
     names = xname || yname;
     run partial(x, y, names, &id, uv, uvname);
     %if &out ^= %str() %then %do;
	  	  nv = char(ncol(xname),2,0);
		  call symput("nv",nv);
        create &out from uv[colname=uvname];
        append from uv[colname=uvname];
     %end;
quit;

*-- Add variable labels to partial variables;
%if &out ^=%str() %then %do;
%put nv = &nv;
data &out;
	set &out;
	label
	%do i = 1 %to &nv;
		%let xname = %scan(&xvar,&i);
		%let xn = &xname;
		%if %length(&xname)>7 %then %let xname=%substr(&xname,1,7);
		U&xname = "Partial &yvar"
		V&xname = "Partial &xn"
	%end; %str(;)
%end;

%mend PARTIAL;
