/*-------------------------------------------------------------------*
 * POISPLOT SAS   Poissonness plot for discrete distributions        *
 *                                                                   *
 *  Hoaglin & Tukey, Checking the shape of discrete distributions.   *
 *   In Hoaglin, Mosteller & Tukey (Eds.), Exploring data tables,    *
 *   trends, and shapes, NY: Wiley, 1985.                            *
 *-------------------------------------------------------------------*
 *  Author:  Michael Friendly            <FRIENDLY@YORKVM1>          *
 * Created:  19 Mar 1991 08:48:26                                    *
 * Revised:  19 Mar 1991 08:48:26                                    *
 * Version:  1.0                                                     *
 *                                                                   *
 *-------------------------------------------------------------------*/
 /*               Copyright (c) 1994, Michael Friendly.               * 
 * Permission to use and distribute this program is granted as long  *
 * as this copyright notice remains intact.  Please acknowledge      *
 * Michael Friendly as author of this program in publications which  *
 * result from its use.                                              * 
 *-------------------------------------------------------------------*/

 %macro poisplot(
     data=_last_,
     count=,            /* basic count variable                     */
     freq=,             /* number of occurrences of count           */
     label=,            /* Horizontal (count) label                 */
     lambda=0);         /* trial value  of lambda to level the plot */
 
%if &label=%str() %then %let label=&count;
proc means data=&data N sum sumwgt mean var;
   var &count;
   weight &freq;
   output out=sum sumwgt=N sum=sum mean=mean;
data poisplot;
   set &data;
   if _n_=1 then set sum(drop=_type_ _freq_);
   k = &count;
   nk= &freq;                     * n(k);
   kf= gamma(k+1);                * k!  ;
 
   *-- if levelling the plot, subtract centering value;
   %if &lambda = 0
       %then %str( level =  0; );
       %else %str( level = &lambda - k * log( &lambda ); );
 
   y =  log(kf * nk / N) + level ;    * poisson metameter;
 
   *-- centered value n*(k), Hoaglin & Tukey, Eqn 9 ;
   p = nk / N;
   if nk >=2 then nkc = nk - .67 - .8*p;
             else nkc = exp(-1);
   yc=  log(kf * nkc/ N) + level ;
 
   *-- half-length of confidence interval for log(eta-k) [Eqn 10];
   h = 1.96 * sqrt( (1-p) / ( nk-((.47+.25*p)*sqrt(nk)) ) );
   ylo = yc - h;
   yhi = yc + h;
proc print data=poisplot;
   id k;
   var nk y nkc yc h ylo yhi;
   sum nk nkc;
   format y yc h yhi ylo 6.3;
 
*-- Calculate goodness of fit chisquare;
data fit;
   set poisplot;
   if _n_=1 then set sum(keep=mean n);
   lambda = mean;
   phat = exp(-lambda) * lambda**k / kf;
   exp  = N * phat;
   chisq= (nk - exp)**2 / exp;
proc print data=fit;
   id k;
   var nk p phat exp chisq;
   sum nk exp chisq;
proc reg data=poisplot outest=parms;
   model y =  k;
data stats;        *-- Annotate data set to label the plot;
   set parms (keep=k intercep);
   set sum   (keep=mean);
   drop k intercep;
   length text $30 function $8;
   xsys='1'; ysys='1';
   x=15;   y=16;
   function = 'LABEL';
   size = 1.4;
   style = 'DUPLEX';
   color = 'RED';
   position='3'; text ='slope =   '||put(k,f6.3);      output;
   position='6'; text ='intercept='||put(intercep,f6.3); output;
   ek = exp(k - &lambda);
   y=10;
   position='6'; text ='lambda:  mean = '||put(mean,5.3); output;
   position='9'; text ='       exp(slope) = '||put(ek,5.3); output;
data conf;
   set poisplot;
   drop yc;
   xsys='2'; ysys='2';
   x = k;    line=33;
   y = yc;   function='MOVE  '; output;
   text='+'; function='SYMBOL'; output;
   y = yhi;  function='DRAW  '; output;
   y = yc;   function='MOVE  '; output;
   y = ylo;  function='DRAW  '; output;
data stats;
   set stats conf;
*-- find range of confidence limits to set y axis extrema;
proc means data=conf noprint;
   var y;
   output out=range min=min max=max;
data _null_;
   set range;
   min = floor(min);
   max = ceil(max);
   call symput('MIN', left(put(min,2.)));
   call symput('MAX', left(put(max,2.)));
run;
proc gplot data=poisplot;
   plot y * k / anno=stats vaxis=axis1 haxis=axis2;
   symbol v=- h=2 i=rl c=black;
   axis1 order=(&min to &max)
         label=(a=90 r=0 f=duplex h=1.4
          'Poisson metameter, ln(k!  n(k) / N)')
         value=(h=1.3);
   axis2 offset=(3) minor=none
         label=(f=duplex h=1.4 "k  (&label)")
         value=(h=1.3);
%mend;
