 /*-------------------------------------------------------------------*
  * SCATMAT SAS     Construct a scatterplot matrix - all pairwise     *
  *                 plots for n variables.                            *
  *                                                                   *
  * %scatmat(data=, var=, group=);                                    *
  *-------------------------------------------------------------------*
  *  Author:  Michael Friendly            <FRIENDLY@YORKVM1>          *
  * Created:  4 Oct 1989 11:07:50                                     *
  * Revised:  4 Sep 1991 11:33:32                                     *
  * Version:  1.2                                                     *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *-------------------------------------------------------------------*/
 
%macro SCATMAT(
        data =_LAST_,          /* data set to be plotted             */
        var  =_NUMERIC_,       /* variables to be plotted - can be   */
                               /* a list or X1-X4 or VARA--VARB      */
        group=,                /* grouping variable (plot symbol)    */
		  interp=none,           /* plot interpolation method          */
        symbols=%str(- + : $ = X _ Y),
        colors=BLACK RED GREEN BLUE BROWN YELLOW ORANGE PURPLE,
        gout=GSEG);            /* graphic catalog for plot matrix    */
 
 options nonotes;
*proc greplay igout=gseg nofs;
*delete _all_;
*quit;

*-- Parse variables list;
 data _null_;
 set &data (obs=1);
    %if %index(&var,-) > 0 or "&var"="_NUMERIC_" %then %do;
 
       * find the number of variables in the list and
         convert shorthand variable list to long form;
     length _vname_ $ 8 _vlist_ $ 200;
     array _xx_ &var;
     _vname_ = ' ';
     do over _xx_;
        call vname(_xx_,_vname_);
        if _vname_ ne "&group" then do;
           nvar + 1;
           if nvar = 1 then startpt = 1;
                       else startpt = length(_vlist_) + 2;
           endpt = length(_vname_);
           substr(_vlist_,startpt,endpt) = _vname_;
        end;
     end;
     call symput( 'VAR', _vlist_ );
put nvar=;
   %end;
   %else %do;
     * find the number of variables in the list;
     nvar = n(of &var) + nmiss(of &var);
put nvar=;
   %end;
   call symput('NVAR',trim(left(put(nvar,2.))));
 RUN;
%put nvar= &nvar;
%if &nvar < 2 or &nvar > 10 %then %do;
   %put Cannot do a scatterplot matrix for &nvar variables ;
   %goto DONE;
   %end;
 
 /*----------------------------------------------------*
  | Determine grouping variable and plotting symbol(s) |
  *----------------------------------------------------*/
%if &group = %str() %then %do;
   %let NGROUPS=1;
   %let plotsym=1;      /* SYMBOL for data panels  */
   %let plotnam=2;      /* for variable name panel */
   %end;
%else %do;
   %let plotsym=&group;
   *-- How many levels of group variable? --;
   proc freq data = &data;
      tables &group / noprint out=_DATA_;
   data _null_;
      set end=eof;
      ngroups+1;
      if eof then do;
         call symput( 'NGROUPS', put(ngroups,3.) );
      end;
    run;
    %let plotnam=%eval(&ngroups+1);
%end;
 
%gensym(n=&ngroups, ht=&nvar, i=&interp, symbols=&symbols, colors=&colors);
%gdispla(OFF);
 
title h=0.1 ' ';
%let plotnum=0;    * number of plots made;
%let replay = ;    * replay list;
 
%do i = 1 %to &nvar;                   /* rows */
   %let vi = %scan(&var , &i );
   proc means noprint data=&data;
      var &vi;
      output out=minmax min=min max=max;
 
   %do j = 1 %to &nvar;                /* cols */
      %let vj = %scan(&var , &j );
      %let plotnum = %eval(&plotnum+1);
      %let replay  = &replay &plotnum:&plotnum ;
      %*put plotting &vi vs. &vj ;
 
      %if &i = &j %then %do;           /* diagonal panel */
         data title;
            length text $8;
            set minmax;
            xsys = '1'; ysys = '1';
            x = 50; y = 50;
            text = "&vi";
            size = 2 * &nvar;
            function = 'LABEL';  output;
 
            x = 6; y = 6; position = '6';
            text = left(put(min, best6.));
            size = &nvar;
            output;
 
            x = 95; y = 95; position = '4';
            text = trim(put(max, best6.));
            size = &nvar;
            output;
 
         proc gplot data = &data;
            plot &vi * &vi = &plotnam
            / frame anno=title vaxis=axis1 haxis=axis1;
         axis1 label=none value=none major=(h=-%eval(&nvar-1))
               minor=none offset=(2);
         run;
      %end;
 
      %else %do;                       /* off-diagonal panel */
         proc gplot data = &data;
            plot &vi * &vj = &plotsym
            / frame nolegend vaxis=axis1 haxis=axis1;
         axis1 label=none value=none major=none minor=none offset=(2);
         run;
      %end;
 
   %end; /* cols */
%end;    /* rows */
 
%gdispla(ON);
 
%macro TDEF(nv, size, shift );
%* ---------------------------------------------------------------;
%* Generate a TDEF statement for a scatterplot matrix             ;
%* Start with (1,1) panel in upper left, and copy it across & down;
%* ---------------------------------------------------------------;
%local i j panl panl1 lx ly;
 
   TDEF scat&nv DES="scatterplot matrix &nv x &nv"
   %let panl=0;
   %let lx = &size;
   %let ly = %eval(100-&size);
   %do i = 1 %to &nv;
   %do j = 1 %to &nv;
       %let panl  = %eval(&panl + 1);
       %if &j=1 %then
          %do;
             %if &i=1 %then %do;      %* (1,1) panel;
               &panl/
                ULX=0  ULY=100   URX=&lx URY=100
                LLX=0  LLY=&ly   LRX=&lx LRY=&ly
                %end;
             %else
                %do;                  %* (i,1) panel;
                   %let panl1 = %eval(&panl - &nv );
               &panl/ copy= &panl1 xlatey= -&shift
                %end;
          %end;
       %else
          %do;
               %let panl1 = %eval(&panl - 1);
               &panl/ copy= &panl1 xlatex= &shift
          %end;
   %end;
   %end;
     %str(;);      %* end the TDEF statement;
%mend TDEF;
 
proc greplay igout=gseg
              gout=&gout  nofs
             template=scat&nvar
             tc=templt ;
%if &nvar = 2 %then %do;
  TDEF scat2 DES="scatterplot matrix 2x2"
           1/ ULX=0  ULY=100   URX=52  URY=100
              LLX=0  LLY=52    LRX=52  LRY=52
           2/ copy=1 XLATEX= 48       /*  Panels are numbered: */
           3/ copy=1 XLATEY=-48                /*    1   2     */
           4/ copy=3 XLATEX= 48;               /*    3   4     */
%end;
 
%if &nvar = 3 %then  %TDEF(&nvar,34,33);
%if &nvar = 4 %then  %TDEF(&nvar,25,25);
%if &nvar = 5 %then  %TDEF(&nvar,20,20);
%if &nvar = 6 %then  %TDEF(&nvar,17,16);
%if &nvar = 7 %then  %TDEF(&nvar,14,14);
%if &nvar = 8 %then  %TDEF(&nvar,13,12);
%if &nvar = 9 %then  %TDEF(&nvar,12,11);
%if &nvar =10 %then  %TDEF(&nvar,10,10);
 
   TREPLAY &replay;
%DONE:
  options notes;
%mend SCATMAT;
 
 
 /*----------------------------------------------------*
  |  Macro to generate SYMBOL statement for each GROUP |
  *----------------------------------------------------*/
%macro gensym(n=1, ht=1.5, i=none,
              symbols=%str(- + : $ = X _ Y),
              colors=BLACK RED GREEN BLUE BROWN YELLOW ORANGE PURPLE);
    %*-- note: only 8 symbols & colors are defined;
    %*--    revise if more than 8 groups (recycle);
  %local chr col k;
  %do k=1 %to &n ;
     %let chr =%scan(&symbols, &k,' ');
     %let col =%scan(&colors, &k, ' ');
     SYMBOL&k H=&HT V=&chr C=&col I=&i;
  %end;
  %let k=%eval(&n+1);
     SYMBOL&k v=none;
%mend gensym;
