 /*-------------------------------------------------------------------*
  | STARS SAS      Star plot of multivariate data                     |
  |                Plots each observation as a star figure with one   |
  |                ray for each variable, ray length proportional to  |
  |                the size of that variable.                         |
  |                                                                   |
  | Reference: Chambers, Cleveland, Kleiner & Tukey, Graphical methods|
  |            for data analysis, Wadsworth, 1983. pp158-162.         |
  |-------------------------------------------------------------------|
  | Author:   Michael Friendly            <FRIENDLY@YORKVM1>          |
  | Created:  31 Mar 1988 00:15:25                                    |
  | Revised:  8 Nov 1991 15:02:27                                     |
  | Version:  1.1                                                     |
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *-------------------------------------------------------------------*/
 
%macro STARS(
        data=_LAST_,       /* Data set to be displayed             */
        var=_NUMERIC_,     /* Variables, as ordered around the     */
                           /* star from angle=0 (horizontal)       */
        id=,               /* Observation identifier  (char)       */
        minray=.1,         /* Minimum ray length, 0<=MINRAY<1      */
        across=5,          /* Number of stars across a page        */
        down=6             /* Number of stars down a page          */
        );
 /*----------------------------------------------------*
  |  Scale each variable to range from MINRAY to 1.0   |
  *----------------------------------------------------*/
 
PROC IML;
  reset;
  use &DATA;
  read all var{&ID} into  &ID;
  read all var{&VAR} into  X[colname=VARS ];
  n = nrow( x);
  min = J( n , 1 ) * X[>< ,];
  max = J( n , 1 ) * X[<> ,];
  c = &MINRAY ;
  X = c + ( 1 - c ) * ( X - min ) / ( max - min );
  create SCALED from X[ rowname=&ID colname=VARS ];
  append from X[ rowname= &ID ];
  quit;
run;
  %put &DATA dataset variables scaled to range &MINRAY to 1;
 
 /*---------------------------------------*
  |  Find out how many variables and obs. |
  *---------------------------------------*/
data _null_;
   file print;
   array p(k) &var ;
   point=1;
   set scaled point=point nobs=nobs;
   do over p;         /* Loop to count variables*/
   end;
   k = k-1;
   put @10 "STARS plots for data set &DATA" /;
   put @10 'Number of variables    = ' k /;
   put @10 'Number of observations = ' nobs /;
   call symput('NV'  , left(put(k,   2.)));
   call symput('NOBS', put(nobs,5.));
   stop;     /* Don't forget this ! */
run;
 /*---------------------------------------------------*
  |  Text POSITIONs corresponding to rays of varying  |
  |  angle around the star                            |
  *---------------------------------------------------*/
proc format;
   value posn     0-22.5 = '6'  /* left, centered  */
               22.6-67.5 = 'C'  /* left, above     */
               67.6-112.5= 'B'  /* centered, above */
              112.6-157.5= 'A'  /* right, above    */
              157.6-202.5= '4'  /* right, centered */
              202.6-247.5= '7'  /* right, below    */
              247.6-292.5= 'E'  /* centered, below */
              other='F';        /* left, below     */
run;
 
 /*------------------------------------------*
  |  Construct ANNOTATE data set to draw and |
  |  label the star for each observation.    |
  *------------------------------------------*/
data stars;
     length function varname $8;
     array p(k) &var ;
 
     retain s1-s&nv c1-c&nv;
     retain cols   &across      /* number of observations per row  */
            rows   &down        /* number of rows per page         */
            xsys   '1'          /* use data percentage coordinates */
            ysys   '1'          /* for both X and Y                */
            lx ly page 0        /* cell X,Y and page  counters     */
            rx ry r;            /* cell radii                      */
     array s(k)  s1-s&nv;              /* sines of angle      */
     array c(k)  c1-c&nv;              /* cosines of angle    */
 
     drop cols rows rx ry cx cy s1-s&nv c1-c&nv &var;
     drop varname showvar;
     *--- precompute ray angles;
     if page=0 then do;
        do k= 1 to &nv;
           ang = 2 * 3.1415926 * (k-1)/&nv;
           s = sin( ang );
           c = cos( ang );
           p = 1.0;                    /* For variable key */
        end;
        x0 = 50; y0 = 50;
        r  = 30;
        size=2;
        text = 'Variable Assignment Key';
        x  = x0; y = 10;
        function = 'LABEL';   output;
 
        showvar=1;
        link DrawStar;                 /* Do variable key  */
        page+1;
        lx = 0;
        ly = 0;
     end;
 
     set scaled end=lastobs;
     label  =&id;
     showvar=0;
 
     *--- set size of one cell;
     if _n_=1 then do;
        rx= (100/cols)/2;
        ry= (100/rows)/2;
        r = .95 * min(rx,ry);
     end;
 
     /* (CX,CY) specify location of lower left corner */
     /*  as percent of data area                      */
     cx  = 100 * (lx) / cols;
     cy  = 100 * ((rows-1)-ly) / rows;
 
     function = 'LABEL';          /* Label the observation centered */
     size = round(r/12,.1);       /* at bottom of the cell          */
     size = min(max(.8,size),2);  /* .8 <= SIZE <= 2                */
     text = &id;
     position='5';
     x =rx+cx; y=2+cy;
     output;
 
     x0 = cx + rx;                     /* Origin for this star */
     y0 = cy + ry;
 
     link drawstar;
     if ( lastobs ) then do;
        call symput('PAGES',trim(left(page)));
        put 'STARS plot will produce ' page 'page(s).';
     end;
     lx + 1;                 /* next column */
     if lx = cols then do;
        lx = 0;
        ly + 1; end;         /* next row    */
     if ly = rows then do;
        lx = 0;
        ly = 0;
        page + 1; end;       /* next page   */
     return;
 
DrawStar:
     *-- Draw star outline;
     do k = 1 to &nv;
        x = x0 + p * r * c;
        y = y0 + p * r * s;
        if k=1 then function = 'POLY';
               else function = 'POLYCONT';
        output;
     end;
 
     *-- draw rays from center to each point;
     *-- label with the variable name if showvar=1;
     do k = 1 to &nv;
        x=x0; y=y0;
        function='MOVE';   output;
        x = x0 + p * r * c;
        y = y0 + p * r * s;
        function = 'DRAW'; output;
        if showvar = 1 then do;
           ang = 2 * 3.1415926 * (k-1)/&nv;
           varname= ' ';
           call vname(p,varname) ;
           text = trim(left(varname));
           position = left(put(180*ang/3.14159,posn.));
           function = 'LABEL';  output;
        end;
     end;
  return;
run;                         /* Force SAS to do it (DONT REMOVE)  */
 
 /*----------------------------------------*
  |  Plot each page with GSLIDE:           |
  |   - Copy observations for current page |
  |   - Draw plot                          |
  |   - Delete page data set               |
  *----------------------------------------*/
%do pg = 0 %to &pages;
   data slide&pg;                    /* Select current page to plot */
      set stars;
      if page = &pg;
 
   proc gslide annotate=slide&pg &GOUT ;  /* Plot current page        */
   title;
   run;
   proc datasets lib=work; delete slide&pg;
%end; /* end of page */
 
%mend STARS;
