 /*-------------------------------------------------------------------*
  * TWOWAY SAS     Analysis of two way tables                         *
  *                                                                   *
  * Macro for analysis of two way tables, including Tukey's           *
  * 1df test for non-additivity, and graphical display of             *
  * additive fit, together with a diagnostic plot for                 *
  * removable non-additivity.                                         *
  *-------------------------------------------------------------------*
  *  Author:  Michael Friendly            <FRIENDLY@YORKVM1>          *
  * Created:  11 Dec 1989 10:51:22                                    *
  * Revised:  4 Sep 1991 11:34:18                                     *
  * Version:  1.2                                                     *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by SAS Institute Inc., Cary, NC, USA       *
  *                                                                   *
  *-------------------------------------------------------------------*/
%macro TWOWAY(
        data=_LAST_,        /* Data set to be analyzed         */
        var=,               /* list of variables: cols of table*/
        id=,                /* row identifier: char variable   */
        response=Response,  /* Label for response on 2way plot */
        plot=FIT DIAGNOSE,  /* What plots to do?               */
        name=TWOWAY,        /* Name for graphic catalog plots  */
        gout=GSEG);         /* Name for graphic catalog        */
 
%if &var = %str() %then %do;
    %put ERROR: You must supply a VAR= variable list for columns;
    %goto DONE;
%end;
%if &id = %str() %then %do;
    %put ERROR: You must supply an ID= character variable;
    %goto DONE;
%end;
%let plot = %upcase(&plot);
 
proc iml;
   reset;
     use &data;
     read all into  y[colname=clabel rowname=&id] var { &var };
     r = nrow( y);
     c = ncol( y);
     rowmean = y[ , :];
     colmean = y[ : ,];
     allmean = y[ :  ];                        * grand mean ;
 
     roweff = rowmean - allmean;                 * row effects;
     coleff = colmean - allmean;                 * col effects;
     data = ( y       || rowmean || roweff ) //
            ( colmean || allmean || 0 ) //
            ( coleff  || 0       || 0 );
 
     rl    = &id    // {'COLMEAN','COLEFF'};
     cl    = clabel || {'ROWMEAN' 'ROWEFF'};
     print , data [ rowname = rl colname = cl ];
 
     jc = j( r , 1);
     jr = j( 1 , c);
     e = y - (rowmean * jr) - (jc * colmean) + allmean;
     print 'Interaction Residuals ',
           e [ rowname = rl colname = cl ];
 
     sse = e[ ## ];
     dfe = ( r - 1 ) # ( c - 1 );
 
     ssrow = roweff[## ,];  ssa = c * ssrow;
     sscol = coleff[ ,##];  ssb = r * sscol;
     product = ( roweff * coleff ) # y;
     d = product[ + ]  / ( ssrow # sscol );
 
     ssnon = ( ( product[ + ] ) ## 2 ) / ( ssrow # sscol );
     sspe = sse - ssnon;
     ss   = ssa   // ssb // sse // ssnon // sspe ;
     df   = (r-1) //(c-1)// dfe //  1    // dfe-1;
 
     ms = ss / df ; mspe=sspe/(dfe-1);
     f  = ms / (ms[{3 3 3 5 5},]);
 
     source=  { "Rows","Cols","Error","Non-Add","Pure Err"};
     srt = "SOURCE   ";
     sst = "   SS    ";
     dft = "   DF    ";
     mst = "   MS    ";
     ft  = "   F     ";
     reset noname;
     print 'ANALYSIS OF VARIANCE SUMMARY TABLE ',
           'with Tukey 1 df test for Non - Additivity ',,
           source[ colname=srt ]
           ss[ colname=sst format=9.3]  df[ colname=dft format=best8. ]
           ms[ colname=mst format=9.3]  f [ colname=ft];
 
     re = ( roweff * jr );
     cf = ( jc * coleff )  + allmean;
 
     compare = ( roweff * coleff ) / allmean;
     compare = shape( e ,0 , 1) ||
               shape( compare ,0 , 1) ||
               shape( re ,0 ,1) ||
               shape( cf ,0, 1);
     vl = { 'RESIDUAL' 'COMPARE' 'ROWEFF' 'COLFIT'};
     create compare from compare[ colname=vl ];
     append from compare;
 
     /* Calculate slope of Residuals on Comparison values */
     /* for possible power transformation                 */
     xy = compare[,{2 1}];
     slope = sum(xy[,1] # xy[,2]) / xy[##,1];
     slope = d || slope || (1-slope);
     print 'D = Coefficient of alpha ( i ) * beta ( j ) ' ,
           'Slope of regression of Residuals on Comparison values',
           '1 - slope = power for transformation',,
           slope[ colname={D Slope Power}];
    ;
 
start twoway;
 /*-------------------------------------------------------------*
  | Calculate points for lines in two-way display of fitted     |
  | value. Each point is (COLFIT+ROWEFF, COLFIT-ROWEFF).        |
  *-------------------------------------------------------------*/
    do i=1 to r;
       clo  = coleff[><]+allmean;
       from = from // (clo-roweff[i] || clo+roweff[i]);
       chi  = coleff[<>]+allmean;
       to   = to   // (chi-roweff[i] || chi+roweff[i]);
       labl = labl || rl[i];
       end;
    do j=1 to c;
       rlo  = roweff[><];
       to   = to   // (coleff[j]+allmean-rlo || coleff[j]+allmean+rlo);
       rhi  = roweff[<>];
       from = from // (coleff[j]+allmean-rhi || coleff[j]+allmean+rhi);
       labl = labl || cl[j];
       end;
 
    /*----------------------*
     | Find large residuals |
     *----------------------*/
    do i=1 to r;
    do j=1 to c;
       if abs(e[i, j]) > sqrt(mspe) then do;
          from = from // ((cf[i,j]-re[i,j])||(cf[i,j]+re[i,j]));
          to   = to   // ((cf[i,j]-re[i,j])||(cf[i,j]+re[i,j]+e[i,j]));
          end;
       end; end;
 
    /*----------------------------------*
     | Start IML graphics               |
     *----------------------------------*/
    %if &sysver < 6 %then %do;
       %let lib=%scan(&gout,1,'.');
       %let cat=%scan(&gout,2,'.');
       %if &cat=%str() %then %do;
           %let cat=&lib;
           %let lib=work;
           %end;
       call gstart gout={&lib &cat}
             name="&name" descript="Two-way plot for dataset &data";
    %end;
    %else %do;     /* Version 6 */
       call gstart("&gout");
       call gopen("&name",1,"Two-way plot for dataset &data");
    %end;
 
    /**--------------------------------**
     | Find scales for the two-way plot |
     **--------------------------------**/
    call gport({10 10, 90 90});
    call gyaxis( {10 10}, 80, from[,2]//to[,2], 5, 0, '5.0') ;
    call gscale( scale2, from[,2]//to[,2], 5);
    call gscript(3, 40, "&Response",'DUPLEX',3) angle=90;
 
    call gscale( scale1, from[,1]//to[,1], 5);
    window = scale1[1:2] || scale2[1:2];
    call gwindow(window);
 
    /*----------------------------------*
     | Draw lines for fit and residuals |
     *----------------------------------*/
    l = nrow(from);
    call gdrawl( from[1:r+c,],   to[1:r+c,])   style=1 color={"BLACK"};
    call gdrawl( from[r+c+1:l,], to[r+c+1:l,]) style=3 color={"RED"};
 
    /*----------------------------------------*
     | Plot row and column labels at margins; |
     *----------------------------------------*/
    xoffset=.04 * (to[<>,1]-to[><,1]);
    yoffset=0;
    do i=1 to r+c;
       if i>r then do;
          yoffset=-.04 * (to[<>,2]-to[><,2]);
          end;
       call gtext(xoffset+to[i,1],yoffset+to[i,2],labl[i]);
    end;
    call gshow;
    call gstop;
finish;
 
    %if %index(&plot,FIT) > 0 %then %do;
    run twoway;
    %end;
quit;
 
data compare;
   set compare;
   fit = colfit + roweff;
   data= fit + residual;
   diff= colfit - roweff;
 
   /* Print values for fit and diagnostic plots */
proc print data=compare;
   var data roweff colfit fit diff residual compare;
%if %index(&plot,PRINT) > 0 %then %do;
proc plot;
     plot data* diff = '+'
          fit * diff = '*'    / overlay;
proc plot;
     plot residual * compare / vpos=45;
%end;
 
%if %index(&plot,DIAGNOSE) > 0 %then %do;
proc gplot data=compare gout=&gout;
     plot residual * compare
        / vaxis=axis1 haxis=axis2
          vminor=1 hminor=1 name="&name" ;
     symbol1 v=- h=1.4 c=black i=rl;
     axis1 label=(a=90 r=0 h=1.5 f=duplex) value=(h=1.3);
     axis2 label=(h=1.5 f=duplex) value=(h=1.3);
     label residual = 'INTERACTION RESIDUAL'
           compare  = 'COMPARISON VALUE';
     run;
%end;
 
%DONE:
%mend TWOWAY;
